#include <list>
#include <string>
#include <gui/rect.h>
#include <util/string.h>
#include <util/message.h>
#include <util/invoker.h>
#include <gui/button.h>
#include <gui/view.h>
#include <gui/point.h>

#include "dirbar.h"

using namespace os;
using namespace std;

DirBar::DirBar( Rect cFrame, const String& cName, Message* pcMessage, uint32 nResizeMask, uint32 nFlags ) : View(cFrame, cName, nResizeMask, nFlags), Invoker()
{
	m_theMessage=pcMessage;
	m_theHandler=NULL;
	m_theLooper=NULL;
	m_CompletePath="";
}

DirBar::~DirBar()
{
	delete m_theMessage;
	ClearPath();
}

void DirBar::SetPath( string path )
{
	string button_path;
	float button_pos=5;

	// Sanity check
	if(path[0]!='/')
		throw string("Not a valid path");

	// Remove old buttons
	ClearPath();

	// Save the complete path
	m_CompletePath=path;

	// Remove the very first separator
	path=string(path, 1, path.size());

	// Create root buton
	AddButton(button_pos, string("/"), string("/"));

	while(path!="")
	{
		// Find next directory splitter "/"
		size_t s=path.find("/");

		// Extract folder
		string label = string(path, 0, s);

		//Remove it from the path
		path=string(path, s+1, path.size());

		// Create nice path for the buttons
		button_path +=  string("/") + label;

		AddButton(button_pos, label, button_path);

		// No more path separator found
		if(s==string::npos)
			break;
	}

	// finally set up messages and targets for all buttons
	SetMessage( m_theMessage );
	SetTarget( m_theHandler, m_theLooper );
}

void DirBar::AddButton( float& button_pos, string label, string path )
{
	// Make message for button
	Message* msg = NULL;

	if( m_theMessage != NULL )
	{
		msg = new Message( *m_theMessage );
		msg->AddString( "file/path", path );
	}

	// Create the button
	Button* button = new Button( Rect(0,0,1,1), label, label, msg );
				
	// Set up position and size of the new button
	Point size = button->GetPreferredSize( false );
	Rect button_geometry;
	button_geometry.left = button_pos;
	button_geometry.right = button_geometry.left + size.x;
	button_geometry.top = 5.0f;
	button_geometry.bottom = button_geometry.top + size.y;
	button->SetFrame(button_geometry);

	// Increase for the next button
	button_pos = button_geometry.right + 5.0f;

	AddChild( button );	

	m_ListOfButtons.push_back( button );
}

string DirBar::GetPath()
{
	return m_CompletePath;
}

void DirBar::ClearPath()
{
	std::list< os::Button* >::iterator it;
	
	// Delete all buttons
	for(it=m_ListOfButtons.begin(); it!=m_ListOfButtons.end(); it++)
	{
		delete (*it);
	}

	m_ListOfButtons.clear();
}

status_t DirBar::SetTarget( const os::Handler* pcHandler, const Looper* pcLooper )
{
	m_theHandler = (os::Handler*) pcHandler;
	m_theLooper = (os::Looper*) pcLooper;

	std::list< os::Button* >::iterator it;

	// Set target for the all button
	for(it=m_ListOfButtons.begin(); it!=m_ListOfButtons.end(); it++)
	{
		(*it)->SetTarget(m_theHandler, m_theLooper);
	}

	return Invoker::SetTarget( m_theHandler, m_theLooper );
}

Point DirBar::GetPreferredSize(bool bLargest)
{
	// Starting at 10,10 for the borders
	Point p(10.0f,10.0f);

	std::list< os::Button* >::iterator it;
	
	// Delete all buttons
	for(it=m_ListOfButtons.begin(); it!=m_ListOfButtons.end(); it++)
	{
		
		p.x+=(*it)->Width();
		p.y= max(p.y, (*it)->Height()+10.0f);
	}


	return p;
}
